<?php

declare(strict_types = 1);

namespace Drupal\entity_ref_filtering_link\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'entity reference label' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_filtered_link_disable",
 *   label = @Translation("Filter Link Disable"),
 *   description = @Translation("Display the label of the referenced entities linking to a view to filter."),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityReferenceFilteredLinkDisableFormatter extends EntityReferenceFilteredLinkFormatter {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $elements['disable'] = [
      '#title' => $this->t('Disable Link for Entity Name'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('disable'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $summary[] = $this->t('Disable link for: @disable', [
      '@disable' => $this->getSetting('disable'),
    ]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $disable = $this->getSetting('disable');
    $disable = !empty($disable) ? explode(",", $disable) : [];

    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
      $label = $entity->label();
      $entityTypeId = $entity->getEntityTypeId();

      $id = $entityTypeId === "taxonomy_term" ? $entity->tid->value : $entity->id();

      if ($id && !in_array($label, $disable)) {
		$url = $this->pathValidator->getUrlIfValid($this->getSetting('view_url'));
        if (!$url) {
          $elements[$delta] = ['#plain_text' => $label];
          continue;
        }
        $options = $url->getOptions();
        switch ($this->getSetting('mode')) {
          case 'id':
            $options['query'][$this->getSetting('argument_name')] = $id;
            break;

          case 'id_multiple':
            $options['query']["{$this->getSetting('argument_name')}[]"] = $id;
            break;

          case 'id_multiple_d9':
            $options['query']["{$this->getSetting('argument_name')}[$id]"] = $id;
            break;

          case 'autocomplete':
            $options['query'][$this->getSetting('argument_name')] = "{$entity->label()} ({$id})";
            break;

          case 'facet':
            $options['query']['f[0]'] = "{$this->getSetting('argument_name')}:{$id}";
            break;

          case 'label':
            $options['query']['f[0]'] = "{$this->getSetting('argument_name')}:{$entity->label()}";
            break;

          case 'label_no_facet':
            $options['query'][$this->getSetting('argument_name')] = $entity->label();
            break;
        }
        $url->setOptions($options);
        $elements[$delta] = [
          '#type' => 'link',
          '#title' => $label,
          '#url' => $url,
          '#options' => $options,
        ];

        if (!empty($items[$delta]->_attributes)) {
          $elements[$delta]['#options'] += ['attributes' => []];
          $elements[$delta]['#options']['attributes'] += $items[$delta]->_attributes;
          // Unset field item attributes since they have been included in the
          // formatter output and shouldn't be rendered in the field template.
          unset($items[$delta]->_attributes);
        }
      }
      else {
        $elements[$delta] = ['#plain_text' => $label];
      }
      $elements[$delta]['#cache']['tags'] = $entity->getCacheTags();
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'disable' => '',
    ] + parent::defaultSettings();
  }

}
